#include <stdint.h>
#include <stdlib.h>
#include <stdio.h>
#include <string>

template<typename T>
class Array {

  private:

    int size1, size2, size3;
    T *elems;

  public:

    Array() : size1(0), size2(0), size3(0), elems(0) {}
    ~Array() { deallocate_(); }

    void dim(int sz1, int sz2 = 1, int sz3 = 1) {
        deallocate_();
        elems = new T [sz1 * sz2 * sz3] ();
        size1 = sz1;
        size2 = sz2;
        size3 = sz3;
    }

    T & operator() (int i1) {
        return elems[i1];
    }

    T & operator() (int i1, int i2) {
        return elems[i1 * size2 + i2];
    }

    T & operator() (int i1, int i2, int i3) {
        return elems[(i1 * size2 + i2) * size3 + i3];
    }

  private:

    void deallocate_() {
      delete[] elems;
      elems = 0;
    }
};
struct ListElement {
    struct ListElement *next;
    struct ListElement *prev;
    /* element data starts here */
};
class LinkedList {
  public:
    struct ListElement *root;
    struct ListElement *current;
    struct ListElement *last;
    int length;
    int index;
    LinkedList() : root(0), current(0), last(0), length(0), index(-1) {};
    virtual struct ListElement *alloc_elem() = 0;
    virtual void del_elem(struct ListElement *elem) = 0;
};
template <typename T>
class TypedList : public LinkedList {
  public:
    static T empty_T;
    struct ListElement *alloc_elem() {
        char *space = new char [sizeof(struct ListElement) + sizeof(T)];
        T *elem = new (space + sizeof(struct ListElement)) T(empty_T);
        return reinterpret_cast<struct ListElement *>(space);
    }
    void del_elem(struct ListElement *elem) {
        T *realelem = reinterpret_cast<T *> (elem + 1);
        realelem->~T();
        delete[] reinterpret_cast<char *>(elem);
    }
};
template <typename T>
T TypedList<T>::empty_T;
#define LIST_CURRENT(list,type) (* (type *) ((list)->current + 1))



static int32_t p_addpackfile(std::string, int32_t);
static int32_t p_closepack();
static int32_t p_createpack(std::string);
#include <unistd.h>
static unsigned char *packfile;
static int32_t packfilesize;
static int packfd = -1;
static void closepack() {
    if (packfd >= 0) {
        close(packfd);
        packfd = -1;
        delete[] packfile;
        packfile = 0;
        packfilesize = 0;
    }
}
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
static ssize_t fullwrite(int fd, unsigned char *buf, size_t count) {
	size_t n = 0;
	while (n < count) {
		ssize_t r = write(fd, buf + n, count - n);
		if (r < 0 && errno == EINTR)
			continue;
		if (r < 0)
			return r;
		if (r == 0)
			return n;
		n += r;
	}
	return n;
}

static int copyfds(int out_fd, int in_fd, size_t count) {
	unsigned char buf[32768];
	size_t n = 0;
	ssize_t r, r2, maxread;
	while (n < count) {
		if (n + sizeof(buf) <= count) {
			maxread = sizeof(buf);
		} else {
			maxread = count - n;
		}
		r = read(in_fd, buf, maxread);
		if (r < 0 && errno == EINTR)
			continue;
		if (r < 0)
			return r;
		if (r == 0)
			return n;
		r2 = fullwrite(out_fd, buf, r);
		if (r2 < 0)
			return r2;
		if (r2 == 0)
			return n;
		n += r;
	}
	return n;
}

static int putword(int fd, uint32_t w) {
	unsigned char s[4];

	s[0] = w;
	s[1] = w >> 8;
	s[2] = w >> 16;
	s[3] = w >> 24;
	return fullwrite(fd, s, 4) == 4;
}

static int addpackfile(const char *name) {
	int fromfd;
	struct stat statbuf;
	int r;

	if (packfd < 0)
		return 0;
	fromfd = open(name, O_RDONLY);
	if (fromfd < 0)
		return 0;
	/* Get file size */
	if (fstat(fromfd, &statbuf) < 0) {
		close(fromfd);
		return 0;
	}

	/* Go to end to add new file */
	lseek(packfd, 0, SEEK_END);

	/* Store without compression, so compressed and uncompressed sizes
	 * will be the same. */
	putword(packfd, statbuf.st_size);
	putword(packfd, statbuf.st_size);

	r = copyfds(packfd, fromfd, statbuf.st_size);
	close(fromfd);
	if (r != statbuf.st_size)
		return 0;
	return 1;
}

static int32_t p_addpackfile(std::string p1, int32_t p2) {
return addpackfile(p1.c_str());

}

static int32_t p_closepack() {
closepack();
return 1;

}

static int32_t p_createpack(std::string p1) {
closepack();
packfd = open(p1.c_str(), O_CREAT|O_TRUNC|O_WRONLY, 0666);
return packfd >= 0;

}


int main() {
	p_createpack("sounds.pak");
	p_addpackfile("blow1.wav", 0);
	p_addpackfile("blow2.wav", 0);
	p_addpackfile("blow3.wav", 0);
	p_addpackfile("blow4.wav", 0);
	p_addpackfile("blow5.wav", 0);
	p_addpackfile("blow6.wav", 0);
	p_addpackfile("blow7.wav", 0);
	p_addpackfile("bow.wav", 0);
	p_addpackfile("chaos.wav", 0);
	p_addpackfile("darkness.wav", 0);
	p_addpackfile("deathcry.wav", 0);
	p_addpackfile("ding.wav", 0);
	p_addpackfile("evasion.wav", 0);
	p_addpackfile("fall.wav", 0);
	p_addpackfile("flame.wav", 0);
	p_addpackfile("gulp.wav", 0);
	p_addpackfile("harp.wav", 0);
	p_addpackfile("holy.wav", 0);
	p_addpackfile("key.wav", 0);
	p_addpackfile("laaa.wav", 0);
	p_addpackfile("moan.wav", 0);
	p_addpackfile("ow.wav", 0);
	p_addpackfile("select.wav", 0);
	p_addpackfile("splash.wav", 0);
	p_addpackfile("success.wav", 0);
	p_addpackfile("wind.wav", 0);
	p_addpackfile("glass.wav", 0);
	p_addpackfile("chicken.wav", 0);
	p_addpackfile("earth.wav", 0);
	p_addpackfile("missed.wav", 0);
	p_addpackfile("hohoho.wav", 0);
	p_addpackfile("frogs.wav", 0);
	p_addpackfile("cannon.wav", 0);
	p_addpackfile("blackcat.wav", 0);
	p_addpackfile("wolf.wav", 0);
	p_addpackfile("jawa.wav", 0);
	p_addpackfile("metal.wav", 0);
	p_addpackfile("horse.wav", 0);
	p_addpackfile("howl.wav", 0);
	p_addpackfile("huh.wav", 0);
	p_addpackfile("whee.wav", 0);
	p_addpackfile("screaming_woman.wav", 0);
	p_addpackfile("roar.wav", 0);
	p_addpackfile("boar.wav", 0);
	p_addpackfile("lightning.wav", 0);
	p_addpackfile("insect.wav", 0);
	p_closepack();
	return 0;
}

